/*=========================================================================

  Copyright 2004 Sandia Corporation.
  Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
  license for use of this work by or on behalf of the
  U.S. Government. Redistribution and use in source and binary forms, with
  or without modification, are permitted provided that this Notice and any
  statement of authorship are reproduced on all copies.

=========================================================================*/

/*========================================================================
 For general information about using VTK and Qt, see:
 http://www.trolltech.com/products/3rdparty/vtksupport.html
=========================================================================*/

/*========================================================================
 !!! WARNING for those who want to contribute code to this file.
 !!! If you use a commercial edition of Qt, you can modify this code.
 !!! If you use an open source version of Qt, you are free to modify
 !!! and use this code within the guidelines of the GPL license.
 !!! Unfortunately, you cannot contribute the changes back into this
 !!! file.  Doing so creates a conflict between the GPL and BSD-like VTK
 !!! license.
=========================================================================*/

#ifndef _GUI_h
#define _GUI_h

#include "visualization-polar.h"

#include "ui_GUI.h"

#include <QMainWindow>

class vtkRenderer;
class vtkEventQtSlotConnect;
class vtkObject;
class vtkCommand;

class GUI : public QMainWindow, public Ui::GUI
{
    Q_OBJECT

public:
    GUI (Simulation &simulation,
         VisualizationType visType,
         long startPoint = 0);
    ~GUI ();

public slots:
    void centerSelectedPoint ();
    void handleNorthPoleTextEdit ();

    void toggleNeighborArcs (bool newState);
    void toggleGlobalMax (bool newState);
    void toggleLocalMax (bool newState);

protected:
    Simulation &mSimulation;
    Visualization *mVisualization;
    TerrainVisualization *mTerrainVisualization;
    SphericalPolarVisualization *mSphericalPolarVisualization;
    DotVisualization *mDotVisualization;
    VisualizationType mVisType;

    void initCallbacks ();
    void updateAll ();
    void updateAfterNorthPoleChange ();
    void do1Step ();
    void postSteps ();
};

#endif // _GUI_h

