/*

NK Visualization

Copyright (c) 2010 Erkin Bahceci <erkin@cs.utexas.edu>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include "nk.h"

// ************* Landscape *****************

Landscape::Landscape (const SimuParameters &params) :
    params (params)
{
    Point::n = params.n;
    Point::kPlus1Consecutive = params.k + 1;
    Point::rightKPlus1Filter = ~(~0 << Point::kPlus1Consecutive);
    Point::rightKFilter = ~(~0 << params.k);
    fitnessTableSize = params.nContexts * 2;  // 2 ^ (k + 1)
    fitnessTable = new double*[params.n];
    for (int dim = params.n - 1; dim >= 0; --dim)
	fitnessTable[dim] = new double[fitnessTableSize];
}

Landscape::~Landscape ()
{
    for (int dim = params.n - 1; dim >= 0; --dim)
	delete[] fitnessTable[dim];
    delete[] fitnessTable;
}

void
Landscape::initRun (unsigned curRun)
{
    if (curRun == 0)
    {
	// start a random sequence for initial landscape
	Simulation::initRandomState(
	    initLandscapeRandomState, params.landscapeseed);

	for (unsigned dim = 0; dim < params.n; dim++)
	    for (long i = 0; i < fitnessTableSize; ++i)
		fitnessTable[dim][i] = erand48(initLandscapeRandomState);
    }
}

void
Landscape::finishRun ()
{
}

/// calculate the fitness of a point
double
Landscape::getFitnessValue (const Point &point)
{
    unsigned n = params.n;
    long values = point.values;
    double fitness = 0.0;

    // Use point values at dimension dim and the following k dimensions
    // as index into fitnessTable
    long valuePlusKBits = values | ((values & Point::rightKFilter) << n);
    fitness += fitnessTable[0][valuePlusKBits & Point::rightKPlus1Filter];
    for (unsigned dim = 1; dim < n; ++dim)
    {
	valuePlusKBits >>= 1;
	fitness += fitnessTable[dim][valuePlusKBits & Point::rightKPlus1Filter];
    }
    fitness /= n;

    return fitness;
}

/// calculate the index into the fitness table
unsigned long
Landscape::fitidx (long value, unsigned context) const
{
    return (value << params.k) + context;
}
