/*

NK Visualization

Copyright (c) 2010 Erkin Bahceci <erkin@cs.utexas.edu>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include "nk.h"

int
main (int argc, char **argv)
{
    string simuCfgFileName ("");
    VisualizationType visType = VisTypeTerrain;
    long startingPoint = 0;
    int argi = 1;

    // figure out simulation config. file
    if (argc > argi)
    {
	if (!strcmp(argv[argi], "--help"))
	{
	    printf ("Usage: %s [--sphere] [--origin point] [simulation config file]\n\
\n\
Example usage:\n\
1) Run with default settings (rectangular visualization, N:10, K:4, seed:0):\n\
%s\n\
\n\
2) Run with simulation configuration file 'sim.cfg' (rectangular visualization):\n\
%s sim.cfg\n\
\n\
3) Run 'sim.cfg' with spherical visualization:\n\
%s --sphere sim.cfg\n\
\n\
4) Run 'sim.cfg' with spherical visualization with 23 as the origin point:\n\
%s --sphere --origin 23 sim.cfg\n", argv[0], argv[0], argv[0], argv[0], argv[0]);

	    exit(1);
	}
	else if (!strcmp (argv[argi], "--sphere"))
	{
	    argi++;
	    visType = VisTypeSphericalPolar;
	}
    }
    if (argc > argi + 1 && !strcmp (argv[argi], "--origin"))
    {
	sscanf (argv[argi+1], "%ld", &startingPoint);
	argi += 2;
    }
    if (argc > argi)
	simuCfgFileName = argv[argi++];

    Simulation simu (simuCfgFileName);
    simu.executeVisualized (argc, argv, visType, startingPoint);

    return 0;
}
