#ifndef NK_H
#define NK_H

#include <stdio.h>
#include <stdlib.h>

#include <iostream>

#include <strings.h>
#include <math.h>
#include <limits.h>
#include <signal.h>

#include <list>
#include <vector>
#include <string>

#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#ifdef foreach
#undef foreach
#endif
#define foreach BOOST_FOREACH

#include "constants.h"

using namespace std;

class Simulation;

// *******************************************

class SimuParameters
{
public:
    SimuParameters (const string &simuCfgFileName);
    ~SimuParameters ();

    unsigned n, k;

    unsigned long nPoints, nContexts;
    long landscapeseed;

    void print ();

private:
    void readFromFile (const string &simuCfgFileName);
};

class Point
{
    friend class MemoryMember;
    friend class Memory;
    friend class Agent;
    friend class Landscape;

public:
    Point () : values (-1) {}
    Point (long values) : values (values) {}
    Point (const Point &point) : values (point.values) {}
    Point (const Point &point, unsigned dim);
    Point (const Point &point, int minJumpDist, int maxJumpDist,
	   unsigned short *searchRandomState);

    unsigned getValue (unsigned dim) const;
    long getHash () const;
    void print (bool goToNextLine = true) const;
    string toBitString () const;
    int hammingDist (long v) const;
    Point &operator= (const Point &otherPoint);
    bool operator== (const Point &otherPoint) const;
    bool operator!= (const Point &otherPoint) const;

private:
    long values;

    static unsigned n;
    static unsigned kPlus1Consecutive;
    static long rightKPlus1Filter;
    static long rightKFilter;
};

class Landscape
{
public:
    Landscape (const SimuParameters &params);
    ~Landscape ();
    void initRun (unsigned curRun);
    void finishRun ();
    double getFitnessValue (const Point &point);
    unsigned long fitidx (long value, unsigned context) const;

protected:
    const SimuParameters &params;
    double **fitnessTable;
    long fitnessTableSize;

    /// random number generator state for initial landscape
    unsigned short initLandscapeRandomState[3];
};

class Simulation
{
    friend class Visualization;
    friend class TerrainVisualization;

public:
    Simulation (const string &simuCfgFileName);
    ~Simulation ();

    void execute ();
    void executeOneStep ();
    void postStep ();
    void executeVisualized (int argc, char *argv[],
                            VisualizationType visType,
			    long startingPoint = 0);
    void initRun ();
    void finishRun ();

    inline const SimuParameters &getParameters () const { return params; }
    Landscape &getLandscape () { return landscape; }

    static string convertNumToBits (long value, int n);
    static void initRandomState(unsigned short *randomState,
                                unsigned long seed);

protected:
    SimuParameters params;
    Landscape landscape;
};


// Utility functions

bool
open_file (const char *filename,// file name
	   const char *mode,    // return the file pointer
	   FILE **fp,           // open mode: "r", "a", "w"
	   bool required);       // whether to exit if cannot open

void
fgl (FILE *fp);

bool
read_till_keyword (FILE * fp,       // pointer to the file
		   const char keyword[],  // the keyword string
		   bool required);   // whether to abort if keyword not found


#endif // NK_H
