#ifndef VISUALIZATIONBASE_H_
#define VISUALIZATIONBASE_H_

#include "nk.h"

#include <QtGui/QLabel>

#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkXYPlotActor.h>
#include <vtkLookupTable.h>

class Visualization
{
public:
    Visualization (Simulation &simulation,
                   VisualizationType visType,
                   QLabel *labelPoint = NULL);
    ~Visualization ();

    virtual void init ();
    virtual void update () {}
    virtual void updateOnRender () {}
    //virtual bool handleClick (int x, int y, long &pointCoords,
    //                          double &pointFitness) { return false; }

    /// Returns true if the click is handled.
    virtual bool handleClick (int x, int y) { return false; }
    virtual void setNumRecentSteps (int numSteps) {}

    void initInteractorCallbacks ();
    vtkRenderer *getRenderer () { return mRenderer; }
    vtkRenderWindow *getRenderWindow () { return mRenWin; }
    VisualizationType getVisualizationType () { return mVisType; }

protected:
    void createBoundingBox (double x1, double x2,
                            double y1, double y2,
                            double z1, double z2);

    Simulation &mSimulation;
    VisualizationType mVisType;

    int mNumDims;

    vtkRenderWindow *mRenWin;
    vtkRenderer *mRenderer;
    vtkLookupTable *mColorTable;

    QLabel *mLabelPoint;
};

#endif /* VISUALIZATIONBASE_H_ */
