#ifndef VISUALIZATIONDOT_H_
#define VISUALIZATIONDOT_H_

#include "visualization-base.h"

#include <vtkPoints.h>
#include <vtkLegendBoxActor.h>


/// Group of dots with the same color and size.
class DotGroup
{
public:
    DotGroup (vtkRenderer *renderer, int numDots, double *coords,
              double colorRed, double colorGreen, double colorBlue,
              double colorAlpha);
    DotGroup (vtkRenderer *renderer, int numDots, double *coords,
              double *colorRGBA);
    ~DotGroup ();

    void update (bool spherical, int numDims = 12);

protected:
    vtkRenderer *mRenderer;
    vtkActor *mActor;
    vtkPoints* mPoints;

private:
    void init (int numDots, double *coords,
	       double colorRed, double colorGreen, double colorBlue,
	       double colorAlpha);
};

class CustomLegendBoxActor : public vtkLegendBoxActor
{
public:
    static CustomLegendBoxActor * New();
    vtkTypeRevisionMacro(CustomLegendBoxActor, vtkLegendBoxActor);

    vtkPolyDataMapper2D *GetSymbolMapper (int i)
    { return this->SymbolMapper[i]; }

protected:
    CustomLegendBoxActor() : vtkLegendBoxActor() {}
    ~CustomLegendBoxActor() {}
};

class DotVisualization : public virtual Visualization
{
public:
    DotVisualization (Simulation &simulation,
		      VisualizationType visType,
		      QLabel *labelPoint = NULL);
    ~DotVisualization ();

    void toggleLocalMax (bool newState);
    void toggleGlobalMax (bool newState);

    void createLegendBox ();
    void removeAllDotGroups ();

    virtual void postUpdateDots () {}

protected:
    void updatePointDots ();
    void addDotsForPointList (const vector<Point> &pointList,
			      int start,
			      int numPoints,
			      DotType dotType,
			      vector<double> &pointCoords);

    virtual void insertPointDot (const Point &point,
				 DotType dotType,
				 vector<double> *chosenList) {}

    CustomLegendBoxActor *mLegendActor;

    DotGroup *mDotGroups[DotTypeNum];
    int mNumDotGroups;

    static double dotColors[DotTypeNum][4];

    bool mGlobalMaxOn;
    bool mLocalMaxOn;

    vector<Point> mLocalMaxPoints;
    Point mGlobalMaxPoint;
};

#endif /* VISUALIZATIONDOT_H_ */
