/*
 * visualization-polar.h
 *
 *  Created on: Dec 2, 2009
 *      Author: erkin
 */

#ifndef VISUALIZATIONPOLAR_H_
#define VISUALIZATIONPOLAR_H_

#include "vtkUnstructuredGrid.h"
#include "vtkGeometryFilter.h"
#include "vtkDataSetSurfaceFilter.h"

#include "visualization.h"

class SphericalPolarData
{
public:
    double pos[3][3], cornerDist[3];

    SphericalPolarData (double pos[][3], double cornerDist[3])
    {
	memcpy (this->pos[0], pos[0], 3 * 3 * sizeof (double));
	for (int i = 0; i < 3; i++)
	    this->cornerDist[i] = cornerDist[i];
    }

    SphericalPolarData (const SphericalPolarData &other)
    {
	memcpy (pos[0], other.pos[0], 3 * 3 * sizeof (double));
	for (int i = 0; i < 3; i++)
	    cornerDist[i] = other.cornerDist[i];
    }
};

class SphericalPolarVisualization :
    public virtual Visualization, public DotVisualization
{
public:
    SphericalPolarVisualization (Simulation &simulation,
                                 long startingPoint = 0,
                                 VisualizationType visType =
                                     VisTypeSphericalPolar);
    ~SphericalPolarVisualization ();

    void init ();

    void update () { update (false); }
    void update (bool reset);
    void updateOnRender ();

    bool handleClick (int x, int y);
    void centerSelectedPoint ();
    void centerPoint (long newNorthPolePoint,
		      bool resetOrientation);
    long northPolePoint () { return mNorthPolePoint; }

protected:
    virtual void computeTerrain (bool initializing = false);
    void addQuad (vtkIdType ind0, vtkIdType ind1,
		  vtkIdType ind2, vtkIdType ind3);
    void createTerrainGrid (bool update = false, bool reset = false);
    void createSelectedPointDot ();
    double getCornerDist (const Point &point, int r, int c, double pos[3]);
    void checkIfMinDistQuad (const Point &point,
                             int r1, int c1,
                             int r2, int c2,
                             int r3, int c3,
                             int r4, int c4,
                             double &minDist,
                             double minDistPos[][3]);
    void checkIfMinDist2Tri (const Point &point,
			     int r1, int c1,
			     int r2, int c2,
			     int r3, int c3,
			     int r4, int c4,
			     DotType dotType,
			     double &minDist,
			     vector<SphericalPolarData> &dataBestOnes);
    void checkIfMinDistTriangle (const Point &point,
				 int r1, int c1,
				 int r2, int c2,
				 int r3, int c3,
				 DotType dotType,
				 double &minDist,
				 vector<SphericalPolarData> &dataBestOnes);
    void postUpdateDots ();
    void insertPointDot (const Point &point,
			 DotType dotType,
			 vector<double> *chosenList);
    void positionPointDot (const Point &point,
			   DotType dotType,
			   double pointPos[3]);
    void pushPointOutward (double inputPointPos[3],
			   double amount,
			   double outputPointPos[3]);

    void rotateByNorthPole (double pos[3], bool reversed = false);
    void selectPoint (vtkIdType pntId,
		      double radius,
		      double theta,
		      double phi);

    vtkActor *mTerrainActor;
    vtkActor *mGridLinesActor;

    vtkCaptionActor2D *mCaptionActor;

    vtkSmartPointer<vtkDataSetSurfaceFilter> mDataSetSurfaceFilter;

    long mNorthPolePoint;
    double mNorthTheta, mNorthPhi;

    double mPrevZeroPos[3];

    double mSelectedTheta, mSelectedPhi;
    int mSelectedPntInd;
    double mSelectionDotCoords[3];

    DotGroup *mSelectedPointGroup;

    vtkUnstructuredGrid *mTerrainGrid;
    vtkPoints *mPoints;
    float *mPointFitnessData;
    vector<long> mNDimPoints; // The points of the NK space
    int mNumGridPoints;

    double mScaleFactor;
    double mScaleZFactor;

    double mBaseRadius;
};

#endif /* VISUALIZATIONPOLAR_H_ */
