#ifndef VISUALIZATION_H_
#define VISUALIZATION_H_

#include "visualization-dot.h"

#include <vtkRenderWindowInteractor.h>
#include <vtkStructuredGrid.h>
#include <vtkStructuredGridGeometryFilter.h>
#include <vtkFollower.h>
#include <vtkAnimationScene.h>
#include <vtkCaptionActor2D.h>
#include <vtkSmartPointer.h>

#define VTK_CREATE_NEW(class, variable)\
  vtkSmartPointer<class> variable = vtkSmartPointer<class>::New();

class TerrainVisualization :
    public virtual Visualization, public DotVisualization
{
public:
    TerrainVisualization (Simulation &simulation,
			  QLabel *labelPoint = NULL);
    ~TerrainVisualization ();

    void toggleNeighborArcs (bool newState);
    void createNeighborArcs ();

    void createSelectedPointDot ();

    void init ();
    virtual void initTerrainCoords ();

    void update ();
    void updateTerrainCoords ();
    void updateGridLineColor ();
    void updateOnRender ();

    bool handleClick (int x, int y);
    virtual void initCamera ();

protected:
    void createTerrainGridLines (vtkStructuredGridGeometryFilter *grid2PolyFilter);
    void pointNDimToGridRowColumn (unsigned long pointCoords,
				   unsigned &r, unsigned &c);
    unsigned long pointNDimToGridIndex (unsigned long pointCoords);
    void postUpdateDots ();
    void insertPointDot (const Point &point,
			 DotType dotType,
			 vector<double> *chosenList);
    void updateMaxPoints ();

    int mNumDimsX;
    int mNumDimsY;
    unsigned mGridResX;
    unsigned mGridResY;

    bool mNeighborArcsOn;
    int mNumArcs;

    unsigned long mSelectedPntGridIndex;

    double mScaleFactor;
    double mScaleZFactor;

    vtkStructuredGrid *mTerrainGrid;
    float *mPointData;
    float *mPointFitnessData;
    unsigned long *mPointNDimCoords;
    long mNumGridPoints;
    double mArcCenterPoint[3];

    double mAvgFitness;

    vtkActor **mArcActors;
    vtkActor *mGridLinesActor;
    vtkActor *mTerrainActor;
    vtkCaptionActor2D *mCaptionActor;

    DotGroup *mSelectedPointGroup;
};

class AnimTerrainVisualization : public TerrainVisualization
{
protected:
    vtkAnimationScene* mScene;
    bool mAnimationDone;

public:
    AnimTerrainVisualization (Simulation &simulation,
                              QLabel *labelPoint = NULL);

    void init ();
    void initTerrainCoords ();
    void stepAnimation1 (double progress);
    void done () { mAnimationDone = true; }
    //bool handleClick (int x, int y);

    void tick1 (double progress);
    void finish1 ();
};

#endif /* VISUALIZATION_H_ */
